package com.chelvc.framework.nacos.config;

import com.alibaba.cloud.nacos.NacosDiscoveryProperties;
import com.alibaba.cloud.nacos.registry.NacosRegistration;
import com.alibaba.cloud.nacos.registry.NacosServiceRegistry;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;

/**
 * 多服务Nacos配置
 * <p>
 * 自动查找工程中所有依赖服务的application.yml配置文件，并将其中的spring.application.name属性值作为服务名称自动注册到Nacos
 *
 * @author Woody
 * @date 2024/1/30
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MultiserverNacosConfigurer implements ApplicationRunner {
    private final NacosServiceRegistry registry;
    private final NacosDiscoveryProperties properties;
    private final ApplicationContext applicationContext;

    @Override
    public void run(ApplicationArguments args) {
        String applicationName = ApplicationContextHolder.getApplicationName(this.applicationContext);
        for (Resource resource : ApplicationContextHolder.getApplicationResources()) {
            // 将打包的服务注册到nacos（排除当前服务）
            String service = ApplicationContextHolder.getApplicationName(resource);
            if (StringUtils.isEmpty(service) || service.equals(applicationName)) {
                continue;
            }
            this.properties.setService(service);
            this.registry.register(new NacosRegistration(null, this.properties, this.applicationContext));
        }
    }
}